import datetime, traceback
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
from adagenes.tools import generate_variant_dictionary


class FATHMMClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.fathmm_src
        self.srv_prefix = config.fathmm_srv_prefix
        self.extract_keys = config.fathmm_keys
        self.info_lines = config.fathmm_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def process_data(self, vcf_lines):
        variant_dc = generate_variant_dictionary(vcf_lines)
        variants = ','.join(variant_dc.values())

        try:
            json_body = req.get_connection(variants, self.url_pattern, self.genome_version)

            for json_obj in json_body:
                if json_obj:
                    #annotations = []

                    if self.qid_key not in json_obj:
                        continue
                    qid = json_obj[self.qid_key]

                    for k in self.extract_keys:
                        if k in json_obj:
                            pass
                            #annotations.append('{}_{}={}'.format(self.srv_prefix, k, json_body[i][k]))

                    try:
                        json_obj.pop('q_id')
                        vcf_lines[qid][self.srv_prefix] = json_obj
                    except:
                        if self.error_logfile is not None:
                            cur_dt = datetime.datetime.now()
                            date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                            print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc(), file=self.error_logfile+str(date_time)+'.log')
                        else:
                            traceback.format_exc()

        except:
            if self.error_logfile is not None:
                print("error processing request: ", variants, file=self.error_logfile+str(date_time)+'.log')
            else:
                print(": error processing variant response: ;", traceback.format_exc())

        return vcf_lines



import io, requests, os


def import_file(infile, onkopus_server_url=None, genome_version=None):
    """
    Imports a variant file as a new patient into the Onkopus database

    :param infile:
    :param onkopus_server_url:
    :param genome_version:
    :return:
    """
    print("Request Onkopus server: ",onkopus_server_url)

    data = {}
    data['data_dir'] = 'loc'
    data['file_src'] = infile
    data["genome_version"] = genome_version

    filename = os.path.basename(infile)
    print("Import file ",filename)

    files = {'file': open(infile,'rb')}

    response = requests.post(onkopus_server_url, data=data, files=files)
    pid = response.json()

    print("new ID generated: ",pid)
    return pid

def annotate_variant_data(pid, onkopus_server_url=None,genome_version=None, module=None):
    """

    :param pid:
    :param onkopus_server_url:
    :param genome_version:
    :return:
    """
    url = onkopus_server_url + "?data_dir=loc&id=" + str(pid) + "&module=" + str(module) + "&genome_version=" + str(genome_version)
    print("Onkopus annotation request: ",url)

    res = requests.get(url)

    print(res)



import datetime, traceback, copy
import adagenes.tools
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes as ag
import onkopus as op


class DGIdbClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.dgidb_src
        self.srv_prefix = config.dgidb_srv_prefix
        self.extract_keys = config.dgidb_keys
        self.info_lines = config.dgidb_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def process_data(self, vcf_linesf, gene_request=False):
        vcf_lines = ag.tools.filter_wildtype_variants(vcf_linesf)

        retransform = False
        if self.genome_version == "hg38":
            qid_list0 = copy.deepcopy(list(vcf_linesf.keys()))
        else:
            qid_dc, qid_list0 = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
            self.genome_version = "hg38"
            retransform = True

        qid_list = []
        genes = []
        variants = []
        qid_gene_name_dc = {}
        for var in qid_list0:
            if retransform is False:
                var_orig = var
            else:
                qid_orig = qid_dc[var]
                var_orig = qid_orig

            if gene_request is False:
                if "UTA_Adapter" in vcf_lines[var_orig]:
                    if "gene_name" in vcf_lines[var_orig]["UTA_Adapter"]:
                        genes.append(vcf_lines[var_orig]["UTA_Adapter"]["gene_name"])
                        variants.append(vcf_lines[var_orig]["UTA_Adapter"]["variant_exchange"])
                        qid_list.append(var)
                        qid_gene_name_dc[vcf_lines[var_orig]["UTA_Adapter"]["gene_name"]] = var_orig
                    else:
                        pass
                elif "UTA_Adapter_gene_name" in vcf_lines[var_orig].keys():
                    genes.append(vcf_lines[var_orig]["UTA_Adapter_gene_name"])
                    variants.append(vcf_lines[var_orig]["UTA_Adapter_variant_exchange"])
                    qid_list.append(var)
                    qid_gene_name_dc[vcf_lines[var_orig]["UTA_Adapter_gene_name"]] = var_orig
                elif "hgnc_gene_symbol" in vcf_lines[var_orig].keys():
                    genes.append(vcf_lines[var_orig]["hgnc_gene_symbol"])
                    variants.append(vcf_lines[var_orig]["aa_exchange"])
                    qid_list.append(var)
                    qid_gene_name_dc[vcf_lines[var_orig]["hgnc_gene_symbol"]] = var_orig
                # elif "INFO" in vcf_lines[var].keys():
                #    pass
                elif "info_features" in vcf_lines[var_orig].keys():
                    # print("INFO ok")
                    # print(vcf_lines[var]["info_features"])
                    if "UTA_Adapter_gene_name" in vcf_lines[var_orig]["info_features"]:
                        genes.append(vcf_lines[var_orig]["info_features"]["UTA_Adapter_gene_name"])
                        variants.append(vcf_lines[var_orig]["info_features"]["UTA_Adapter_variant_exchange"])
                        qid_list.append(var)
                        qid_gene_name_dc[vcf_lines[var_orig]["info_features"]["UTA_Adapter_gene_name"]] = var_orig
            else:
                genes.append(var)

        qid_lists_query = ag.tools.split_list(qid_list)
        genes_lists_query = ag.tools.split_list(genes)
        variants_lists_query = ag.tools.split_list(variants)

        print("ALIST ",qid_lists_query)

        for q_list in genes_lists_query:
            q = ",".join(q_list)
            query = 'genes=' + q
            query = '?' + query

            try:
                json_body = req.get_connection(query, self.url_pattern, "hg38")

                for gene_name in json_body.keys():

                        if gene_request is False:
                            qid = qid_gene_name_dc[gene_name]
                        else:
                            qid = gene_name

                        try:
                            vcf_lines[qid][self.srv_prefix] = json_body[gene_name]
                        except:
                            if self.error_logfile is not None:
                                cur_dt = datetime.datetime.now()
                                date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                                print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc(), file=self.error_logfile+str(date_time)+'.log')
                            else:
                                print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc())
            except:
                print(": error processing variant response: ;", traceback.format_exc())


        return vcf_lines



import traceback, datetime, json, copy
import urllib.parse
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes as ag


class MolecularFeaturesClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= config.molecular_features_info_lines
        self.url_pattern = config.molecular_features_src
        self.srv_prefix = config.molecular_features_srv_prefix
        self.response_keys = config.molecular_features_response_keys
        self.extract_keys = config.molecular_features_keys

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

    def process_data(self, biomarker_data, tumor_type=None):
        """

        :param biomarker_data:
        :param tumor_type:
        :return:
        """
        try:
            vcf_linesf = ag.tools.filter_wildtype_variants(biomarker_data)

            retransform = False
            if self.genome_version == "hg38":
                qid_list0 = copy.deepcopy(list(vcf_linesf.keys()))
            else:
                qid_dc, qid_list0 = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
                self.genome_version = "hg38"
                retransform = True

            qid_list = []
            genes = []
            variants = []

            #print("len vars ",len(list(biomarker_data.keys())))

            for var in qid_list0:
                if retransform is False:
                    var_orig = var
                else:
                    qid_orig = qid_dc[var]
                    var_orig = qid_orig

                if "UTA_Adapter" in biomarker_data[var_orig]:
                    if "gene_name" in biomarker_data[var_orig]["UTA_Adapter"]:
                        genes.append(biomarker_data[var_orig]["UTA_Adapter"]["gene_name"])
                        variants.append(biomarker_data[var_orig]["UTA_Adapter"]["variant_exchange"])
                        qid_list.append(var)
                    else:
                        pass
                elif "UTA_Adapter_gene_name" in biomarker_data[var_orig].keys():
                    genes.append(biomarker_data[var_orig]["UTA_Adapter_gene_name"])
                    variants.append(biomarker_data[var_orig]["UTA_Adapter_variant_exchange"])
                    qid_list.append(var)
                elif "hgnc_gene_symbol" in biomarker_data[var_orig].keys():
                    genes.append(biomarker_data[var_orig]["hgnc_gene_symbol"])
                    variants.append(biomarker_data[var_orig]["aa_exchange"])
                    qid_list.append(var)
                #elif "INFO" in biomarker_data[var].keys():
                #    pass
                elif "info_features" in biomarker_data[var_orig].keys():
                    #print("INFO ok")
                    #print(biomarker_data[var]["info_features"])
                    if (("UTA_Adapter_gene_name" in biomarker_data[var_orig]["info_features"]) and
                            ("UTA_Adapter_variant_exchange" in biomarker_data[var_orig]["info_features"])):
                        try:
                            genes.append(biomarker_data[var_orig]["info_features"]["UTA_Adapter_gene_name"])
                            variants.append(biomarker_data[var_orig]["info_features"]["UTA_Adapter_variant_exchange"])
                            qid_list.append(var)
                        except:
                            print(traceback.format_exc())
                    #else:
                    #    print("hmm ",biomarker_data[var_orig])
                #else:
                #    print("HMM","keys ",biomarker_data[var])

            variants_enc = []
            for var in variants:
                #var_encoded = urllib.parse.quote(var, safe='=&')
                var_encoded = urllib.parse.quote(var)
                variants_enc.append(var_encoded)
            variants = variants_enc
            #print("variants ", len(variants),variants)

            qid_lists_query = ag.tools.split_list(qid_list)
            genes_lists_query = ag.tools.split_list(genes)
            variants_lists_query = ag.tools.split_list(variants)

            for qlist, glist, vlist in zip(qid_lists_query, genes_lists_query, variants_lists_query):
                q_genes = ",".join(glist)
                q_variants = ",".join(vlist)
                q_genompos = ",".join(qlist)
                q = "?genompos=" + q_genompos + "&gene=" + q_genes + "&variant=" + q_variants

                res = req.get_connection(q, self.url_pattern, self.genome_version)

                for var in res.keys():
                    if isinstance(res[var], dict):
                        if "molecular_features" in res[var]:
                            if retransform is False:
                                #biomarker_data[var]["molecular_features"] = res[var]["molecular_features"]
                                biomarker_data[var][self.srv_prefix] = res[var]["molecular_features"]
                            else:
                                qid_orig = qid_dc[var]
                                biomarker_data[qid_orig][self.srv_prefix] = res[var]["molecular_features"]

        except:
            if self.error_logfile is not None:
                cur_dt = datetime.datetime.now()
                date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                print("error processing request: ", biomarker_data, file=self.error_logfile+str(date_time)+'.log')
            else:
                print(": error processing variant response: ;", traceback.format_exc())

        return biomarker_data



import unittest, os
import onkopus as op


class TestWriteAnnotatedData(unittest.TestCase):

    def test_write_annotated_data(self):
        data = { "chr7:140753336A>T": {} }
        data = op.UTAAdapterClient(genome_version="hg38").process_data(data)
        data = op.DBNSFPClient(genome_version="hg38").process_data(data)

        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        outfile = __location__ + "/../test_files/chr7_140753336AT.csv"
        print("write file")
        op.write_file(outfile,data)

        file = open(outfile)
        contents = file.read()
        contents_expected = """genomic_location_hg38,chrom,pos_hg38,pos_hg19,ref,alt,mutation_type,hgnc_gene_symbol,aa_exchange,aa_exchange_long,ncbi_transcript_mane_select,ncbi_cdna_string,ncbi_cds_start,ncbi_cds_end,ncbi_cds_strand,ncbi_prot_location,ncbi_protein_id,clinvar_clinical_significance,clinvar_review_status,clinvar_cancer_type,clinvar_id,dbsnp_population_frequency,dbsnp_rsid,gnomAD_exomes_ac,gnomAD_exomes_af,1000genomes_af,1000genomes_ac,alfa_total_af,alfa_total_ac,ExAC_AF,ExAC_AC,revel_score,alphamissense_score,mvp_score,loftool_score,vuspredict_score,missense3D_pred,CADD_score_raw,Polyphen2_HDIV_score,Polyphen2_HDIV_pred,Polyphen2_HVAR_score,Polyphen2_HVAR_pred,SIFT_score,SIFT_pred,GERP++_score,MetaLR_score,MetaSVM_score,phastCons17way_primate_score,phyloP17way_primate,MutationAssessor_score,MutationTaster_score,fathmm-MKL_coding_score,fathmm-XF_coding_score,uniprot_id,alphamissense_class,Interpro_domain,protein_sequence_MANE_Select,Secondary_protein_structure,RelASA,BLOSUM62
,7,140753336,,A,T,,BRAF,V600E,Val600Glu,NM_004333.6,,,,,,,,,,,,,1,3.979940e-06,.,.,.,.,1.647e-05,2,,,,,,,4.2,0.97,D (probably damaging),0.94,D (probably damaging),0.0,,5.65,0.2357,-0.7685,0.999000,0.750000,0.65,1.0,0.99,0.91,,,|Serine-threonine/tyrosine-protein kinase  catalytic domain||Protein kinase domain||Protein kinase domain;Serine-threonine/tyrosine-protein kinase  catalytic domain||Protein kinase domain||Protein kinase domain;Serine-threonine/tyrosine-protein kinase  catalytic domain||Protein kinase domain||Protein kinase domain;Serine-threonine/tyrosine-protein kinase  catalytic domain||Protein kinase domain||Protein kinase domain|,,,,\n"""
        self.assertEqual(contents, contents_expected, "")
        file.close()



import unittest, copy
import onkopus.onkopus_clients
import adagenes, os


class LiftoverAnnotationTestCase(unittest.TestCase):

    def test_liftover_client(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        genome_version = 'hg19'

        #infile = "../test_files/somaticMutations.vcf"
        infile = __location__ + "/../../test_files/somaticMutations.vcf"
        outfile = __location__ + "/../../test_files/somaticMutations.tsv.liftover"
        data = adagenes.VCFReader(genome_version).read_file(infile)

        data.data = onkopus.onkopus_clients.LiftOverClient(
            genome_version=genome_version).process_data(data.data)

        print("Response ",data.data)






import unittest, copy
import onkopus.onkopus_clients


class DBSNPAnnotationTestCase(unittest.TestCase):

    def test_dbsnp_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr1:2556710C>A": {},
                "chr1:2556710C>T":{},"chr1:2556714A>G":{}, "chr1:2556718C>T":{},
                "chr1:2556718C>.": {}, "chr1:2556710C>.":{}
                }

        variant_data = onkopus.onkopus_clients.DBSNPClient(
            genome_version=genome_version).process_data(data)

        print("Response ",variant_data)
        self.assertListEqual(["chr7:140753336A>T", "chr1:2556710C>A",
                "chr1:2556710C>T","chr1:2556714A>G", "chr1:2556718C>T",
                "chr1:2556718C>.", "chr1:2556710C>."], list(variant_data.keys()),
                             "")
        self.assertEqual('0:23038(0.0)', variant_data["chr1:2556710C>T"]["dbsnp"]["freq_total"], "")




import unittest, os
import onkopus as op
import adagenes as ag

class TestStreamAnnotation(unittest.TestCase):

    def test_stream_vcf_liftover(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + '/../test_files/somaticMutations.ln50.vcf'
        outfile = __location__ + "/../test_files/somaticMutations.hg38.ln50.lo.vcf"

        client = op.LiftOverClient(genome_version="hg19", target_genome="hg38")
        ag.process_file(infile,outfile,client)

        with open(outfile, 'r') as file:
            contents = file.read()

        cont_expected = ('Adapter-LiftOver,Number=1,Type=String,Description="Reference Genome '
 'LiftOver">\n'
 '#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n'
 'chr7\t21744593\t.\tA\tAG\t.\t.\t.\n'
 'chr10')

        self.assertEqual(contents[50:200], cont_expected, "")

    def test_stream_vcf_annotate_clinvar(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + '/../test_files/somaticMutations.hg38.ln50.vcf'
        outfile = __location__ + "/../test_files/somaticMutations.hg38.ln50.clinvar.out.vcf"

        client = op.ClinVarClient(genome_version="hg38")
        ag.process_file(infile,outfile,client,genome_version="hg38")

        #with open(outfile, 'r') as file:
        #    contents = file.read()

        #self.assertEqual(contents[50:400],
        #                 'Adapter-LiftOver,Number=1,Type=String,Description="Reference Genome '
        #                 'LiftOver">\n'
        #                 'chr#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO;\n'
        #                 'chr7\t21744592\t.\tA\tAG\t.\t.\t.\n'
        #                 'chr10\t8073950\t.\tC\tT\t.\t.\t.\n'
        #                 'chr17\t7778426\t.\tT\tC\t.\t.\t.\n'
        #                 'chr1\t148998357\t.\tG\tA\t.\t.\t.\n'
        #                 'chr1\t237638364\t.\tG\tA\t.\t.\t.\n'
        #                 'chr5\t13882966\t.\tC\tT\t.\t.\t.\n'
        #                 'chr7\t92029936\t.\tA\tG\t.\t.\t.\n'
        #                 'chr11\t62520346\t.\tC\tT\t.\t.\t.\n'
        #                 'chr11\t62524581\t.'
        #                 , "")



import unittest
import onkopus.onkopus_clients
import adagenes as ag


class UTAAdapterProteinSequenceAnnotationTestCase(unittest.TestCase):

    def test_pseq_client_hg19(self):
        genome_version = 'hg19'
        data = {"chr7:140453136A>T": {}}
        variant_data = ag.LiftoverAnnotationClient(genome_version=genome_version,target_genome="hg38").process_data(data)
        variant_data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(variant_data)
        variant_data = onkopus.onkopus_clients.UTAAdapterProteinSequenceClient(
            genome_version=genome_version).process_data(variant_data)
        print("Response ", variant_data)
        self.assertEqual(variant_data["chr7:140453136A>T"]["UTA_Adapter_protein_sequence"]["protein_sequence"][:18],
                         "MAALSGGGGGGAEPGQAL", "")

    def test_uta_adapter_protein_sequence_client_variant(self):
        genome_version = 'hg38'
        data = {"chr7:140753336A>T": {}, "chr10:8115913C>T":{},"chr7:140753336A>G": {}}
        variant_data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        variant_data = onkopus.onkopus_clients.UTAAdapterProteinSequenceClient(
            genome_version=genome_version).process_data(variant_data)
        print("Response ",variant_data)
        self.assertEqual(variant_data["chr7:140753336A>T"]["UTA_Adapter_protein_sequence"]["protein_sequence"][:18],"MAALSGGGGGGAEPGQAL","")

    def test_uta_adapter_protein_sequence_client_gene(self):
        genome_version = 'hg38'
        data = {"BRAF": {}, "NRAS": {}}
        variant_data = onkopus.onkopus_clients.UTAAdapterProteinSequenceClient(
            genome_version=genome_version).process_data(data,gene_request=True)
        print("Response ",variant_data)
        self.assertListEqual(list(variant_data.keys()),["BRAF","NRAS"],"")




import argparse
import os, time, datetime
import onkopus.conf.docker_config as docker_config
import onkopus.conf.read_config as conf_reader
import adagenes as ag
import onkopus.import_data
import onkopus.mtb_requests
import onkopus as op
import onkopus.cli.run_interpretation
import onkopus.cli.module_mgt


class OnkopusMain():

    def __init__(self):
        self.__location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        self.__data_dir__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__))) + "/../data"

        self.title = "Onkopus"

        version = op.conf_reader.config["DEFAULT"]["VERSION"]
        self.version = version
        self.__available_modules__ = self._load_available_modules()
        self.module_labels = docker_config.available_modules

        # Load installed modules
        self.installed_modules = self._load_installed_modules()
        self.__docker_compose_file = self.__data_dir__ + "/docker-compose.yml"

        self.formats=["VCF","MAF","CSV","TSV","XLSX","TXT"]

    def _load_available_modules(self):
        return list(docker_config.module_ids.keys())

    def _load_installed_modules(self):
        installed_modules = []

        # Create data directory if it does not exist
        isExist = os.path.exists(self.__data_dir__)
        if not isExist:
            # Create a new directory because it does not exist
            os.makedirs(self.__data_dir__)
            print("Created data directory: ",self.__data_dir__)

        # Create modules file if it does not exist
        if not os.path.exists(self.__data_dir__ + "/" + conf_reader.config["DEFAULT"]["INSTALLED_MODULES_FILE"]):
            file = open(self.__data_dir__ + "/" + conf_reader.config["DEFAULT"]["INSTALLED_MODULES_FILE"], 'w+')
            file.close()
        if not os.path.exists(self.__data_dir__ +
                              "/docker-compose.yml"):
            os.system("cp -v " + self.__location__ + "/conf/docker_compose/docker-compose.yml " + self.__data_dir__ + "/docker-compose.yml")

        file = open(self.__data_dir__ + "/" + conf_reader.config["DEFAULT"]["INSTALLED_MODULES_FILE"], 'r')
        for line in file:
            installed_modules.append(line.strip())
        file.close()

        print("Installed modules: ", installed_modules)

        return installed_modules


    def import_data(self,input_file, genome_version=None,data_dir=None):
        """
        Imports an input file or all files within a directory as patient biomarker data in the Onkopus database

        :param input_file:
        :param genome_version:
        :param data_dir:
        :return:
        """
        if (data_dir is not None) and (data_dir != ''):
            # import all files in directory
            pass
        elif input_file != '':
            # import file
            print("Import file: ",input_file)

            onkopus_server_url = conf_reader.__MODULE_PROTOCOL__ + "://" + conf_reader.__MODULE_SERVER__ + conf_reader.__PORT_ONKOPUS_SERVER__
            onkopus_server_url = onkopus_server_url + "/onkopus-server/v1/upload_variant_data"

            pid = onkopus.import_data.import_file(input_file, onkopus_server_url=onkopus_server_url, genome_version=genome_version)
            return pid
        else:
            print("Error: No input file defined. Define an input file or a data directory")

    def restart_modules(self):
        """
        Restarts all installed Onkopus modules

        :return:
        """
        os.system('docker-compose down')
        os.system('docker-compose pull')
        os.system('docker-compose up -d')

    def list_modules(self):
        """
        Prints a list of all available Onkopus modules

        :return:
        """
        print("Available Onkopus modules: ")
        #print(self.__available_modules__)
        print(self.module_labels)

        print("Install an Onkopus module locally by running 'onkopus install -m [module-name]'")

    def list_formats(self):
        """

        :return:
        """
        print("Available data formats for input format (-if) and output format (-of): ")
        for format in self.formats:
            print(format)

    def add_patient_to_mtb(self,pid,mtb):
        onkopus_server_url = conf_reader.__MODULE_PROTOCOL__ + "://" + conf_reader.__MODULE_SERVER__ + conf_reader.__PORT_ONKOPUS_SERVER__
        onkopus_server_url = onkopus_server_url + "/onkopus-server/v1/updateMTB"
        onkopus.mtb_requests.add_patient_to_mtb(pid,mtb, onkopus_server_url)

    def add_patient_and_perform_annotation(self, input_file, mtb, genome_version=None, data_dir=None):
        pid = self.import_data(input_file, genome_version=genome_version, data_dir=data_dir)
        module = 'all'
        data_dir = "loc"
        self._db_request(pid, module=module, genome_version=genome_version)
        self.add_patient_to_mtb(pid, mtb)

    def show_title(self):
        pass

    def run(self):
        """
        Main Onkopus command-line function

        :return:
        """
        self.show_title()

        parser = argparse.ArgumentParser()
        parser.add_argument('action', choices=['run', 'install', 'list-modules', 'start', 'stop', 'import', 'mtb-add', 'mtb-add-analyze'])
        parser.add_argument('-m', '--module', default='all')
        parser.add_argument('-i', '--input_file', default='')
        parser.add_argument('-o', '--output_file', default='')
        parser.add_argument('-g', '--genome_version', default='')
        parser.add_argument('-d', '--data_dir', default='')
        parser.add_argument('-pid', '--patient_id', default='')
        parser.add_argument('-mtb', '--mtb_id', default='')
        parser.add_argument('-if', '--input_format', default='')
        parser.add_argument('-of', '--output_format', default='')
        parser.add_argument('-md', '--mode', default='')
        parser.add_argument('-t', '--target', default='hg38')

        args = parser.parse_args()
        action = args.action
        module = args.module
        input_file = args.input_file
        output_file = args.output_file
        genome_version = args.genome_version
        data_dir = args.data_dir
        pid = args.patient_id
        mtb = args.mtb_id
        input_format = args.input_format
        output_format = args.output_format
        mode = args.mode
        target = args.target

        if action == 'run':
            onkopus.cli.run_interpretation.run_interpretation(input_file, output_file, module=module, genome_version=genome_version, pid=pid,
                                    input_format=input_format,output_format=output_format,mode=mode, target=target)
        elif action == 'install':
            onkopus.cli.module_mgt.install(module, self.installed_modules, self.__available_modules__)
        elif action == 'start':
            onkopus.cli.module_mgt.start_modules(self.installed_modules)
        elif action == 'stop':
            onkopus.cli.module_mgt.stop_modules()
        elif action == 'import':
            self.import_data(input_file, genome_version=genome_version,data_dir=data_dir)
        elif action == 'list-modules':
            self.list_modules()
        elif action == 'list-formats':
            self.list_formats()
        elif action == 'mtb-add':
            self.add_patient_to_mtb(pid,mtb)
        elif action == 'mtb-add-analyze':
            self.add_patient_and_perform_annotation(input_file, mtb, genome_version=genome_version, data_dir=data_dir)




import unittest
import onkopus.onkopus_clients

class PrimateAIAnnotationTestCase(unittest.TestCase):

    def test_primateai_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        data = onkopus.onkopus_clients.UTAAdapterClient(genome_version=genome_version).process_data(data)
        data = onkopus.onkopus_clients.LiftOverClient(genome_version=genome_version).process_data(data)
        variant_data = onkopus.onkopus_clients.PrimateAIClient(
            genome_version=genome_version).process_data(data)

        print("Response ",variant_data)

    #def test_primateai_client_batch(self):
    #    genome_version = 'hg19'
    #    file='../test_files/somaticMutations.l100.vcf'
    #    data = onkopus.read_file(file, genome_version=genome_version)
    #    data.data = onkopus.onkopus_clients.UTAAdapterClient(genome_version=genome_version).process_data(data.data)
    #    data.data = onkopus.onkopus_clients.LiftOverClient(genome_version=genome_version).process_data(data.data)
    #    data.data = onkopus.onkopus_clients.PrimateAIClient(
    #        genome_version=genome_version).process_data(data.data)
    #    print(data.data)



import unittest
import onkopus as op
import adagenes as ag


class TestInDelAnnotationPipeline(unittest.TestCase):

    def test_indel_annotation(self):
        data = {"chr16:68846077C>CTTCAA":{}}
        data = op.indel_request(data)
        #print(data["chr16:68846077C>CTTCAA"]["clinvar"])
        print(data)

        #self.assertListEqual(list(data["chr16:68846077C>CTTCAA"].keys()),["variant_data","gencode_genomic","clinvar"],"Sections do not match")
        self.assertEqual(data["chr16:68846078_68846082insTTCAA"]["gencode_genomic"]["MANE_Select_transcript"],"ENST00000261778.2","")
        self.assertEqual(data["chr16:68846078_68846082insTTCAA"]["UTA_Adapter_indel"]["gene_name"],
                         "TANGO6", "")

    def test_short_indel(self):
        qid = "chr4:1802095insGGG"
        bframe = ag.BiomarkerFrame(qid)
        data = op.annotate(bframe)
        print(data)



import traceback, datetime, json
import onkopus as op
from onkopus.conf import read_config as config


class PathogenicityClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= config.onkopus_aggregator_info_lines
        self.url_pattern = config.onkopus_aggregator_src
        self.srv_prefix = config.onkopus_aggregator_srv_prefix
        self.response_keys = config.onkopus_aggregator_response_keys
        self.extract_keys = config.onkopus_aggregator_keys

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

    def process_data(self, biomarker_data):

        # dbNSFP
        biomarker_data = op.DBNSFPClient(self.genome_version).process_data(biomarker_data)

        # REVEL
        biomarker_data = op.REVELClient(self.genome_version).process_data(biomarker_data)

        # AlphaMissense
        biomarker_data = op.AlphaMissenseClient(self.genome_version).process_data(biomarker_data)

        # LoFTool
        biomarker_data = op.LoFToolClient(self.genome_version).process_data(biomarker_data)

        return biomarker_data



import unittest
import adagenes as ag
import onkopus as op


class GETestCase(unittest.TestCase):

    def test_ge_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr12:25245350C>T": {}, "BRAF":{}}
        bframe = ag.BiomarkerFrame(data)
        print("gen bframe ",bframe)

        variant_data = op.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(bframe.data)
        print("uta response", variant_data)

        variant_data = op.GeneExpressionClient().process_data(bframe.data)
        print(variant_data["chr7:140753336A>T"]["gtex"])
        self.assertEqual(variant_data["chr7:140753336A>T"]["gtex"]['Adipose_Subcutaneous'], 7.77011, "")




import unittest
import onkopus as op
import adagenes as ag


class TestGeneAnnotation(unittest.TestCase):

    def test_gene_annotation(self):
        data = {"TP53":{ "mutation_type": "gene" }}
        bframe = ag.BiomarkerFrame(data)
        data = op.annotate_genes(bframe.data)
        print(data["TP53"].keys())
        #print(data)
        self.assertEqual(set(list(data["TP53"].keys())),
                         set(['mutation_type', 'type', 'mdesc', 'cosmic', 'civic', 'dgidb', 'gencode', 'onkopus_aggregator', 'UTA_Adapter']),
                         "")




import traceback, copy
import adagenes as ag
from onkopus.conf import read_config as config


class LiftOverClient:

    def __init__(self, genome_version, target_genome=None, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines = config.uta_adapter_liftover_info_lines
        self.url_pattern = config.uta_adapter_liftover_src
        self.srv_prefix = config.uta_adapter_liftover_srv_prefix
        self.genomic_keys = config.uta_genomic_keys
        self.gene_keys = config.uta_liftover_gene_keys
        self.gene_response_keys = config.uta_liftover_response_keys
        self.extract_keys = config.uta_liftover_gene_keys
        self.target_genome = target_genome

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def process_data(self, vcf_lines, target_genome="hg38", input_format='json'):
        return ag.LiftoverClient(genome_version=self.genome_version,target_genome=self.target_genome).process_data(vcf_lines, genome_version=self.genome_version, target_genome=target_genome)




from adagenes.plot.generate_plots.generate_sunburst_plots import generate_sunburst_plot
import adagenes.plot.generate_data.generate_radarplot_data
import adagenes.plot.generate_plots.generate_radarplot
import adagenes as ag
import onkopus as op
import onkopus.onkopus_clients


def generate_plot_graph_data(variant_data, qid, plot_type, width=None, height=None, genome_version=None,mutation_type=None, font_size=18):
    """
    Generates plot data for all Onkopus plots implemented in Plotly. Returns the JSON graph to display the plot

    :param variant_data:
    :param qid:
    :param plot_type:
    :param width:
    :param height:
    :return:
    """
    if mutation_type is not None:
        if mutation_type in variant_data.keys():
            variant_data = variant_data[mutation_type]
    #if "snvs" in variant_data:
    #    variant_data = variant_data["snvs"]

    if plot_type == "pathogenicity-scores-radar":
        #df = adagenes.plot.generate_data.generate_radarplot_data.generate_single_biomarker_radar_plot_data(variant_data,qid)
        #dfs=[]
        #dfs.append(df)
        #names = adagenes.plot.generate_data.generate_biomarker_identifiers(variant_data)
        #if (width is not None) and (height is not None):
        #    return adagenes.plot.generate_plots.generate_radarplot.generate_radar_plot(dfs,names,
        #                                                    "pathogenicity-scores-radar", width=width,height=height)
        #else:
        #    return adagenes.plot.generate_plots.generate_radarplot.generate_radar_plot(dfs, names,
        #                                                                             "pathogenicity-scores-radar")
        return onkopus.onkopus_clients.PlotClient(genome_version="hg38").process_data(variant_data,
                                                                                      plot="pathogenicity-scores-radar")
    elif plot_type == 'pathogenicity-scores-radar-set':
        #dfs = adagenes.plot.generate_data.generate_radarplot_data.generate_multiple_biomarker_radar_plot_data(variant_data)
        #names=adagenes.plot.generate_data.generate_biomarker_identifiers(variant_data)
        #if (width is not None) and (height is not None):
        #    graph = adagenes.plot.generate_plots.generate_radarplot.generate_radar_plot\
        #        (dfs,names, "pathogenicity-scores-radar",width=width,height=height)
        #    return graph
        #else:
        #    return adagenes.plot.generate_plots.generate_radarplot.generate_radar_plot(dfs, names,
        #                                                                             "pathogenicity-scores-radar")
        return onkopus.onkopus_clients.PlotClient(genome_version="hg38").process_data(variant_data,
                                                                                      plot="multi_pathogenicity")
    elif plot_type == 'pathogenicity-scores-radar-categorical':
        #dfs = adagenes.plot.generate_data.generate_radarplot_data.generate_multiple_biomarker_radar_plot_data(variant_data)
        #names = adagenes.plot.generate_data.generate_biomarker_identifiers(variant_data)
        #df_cat = adagenes.plot.generate_data.generate_radarplot_data.generate_categorical_scores(dfs)
        #if (width is not None) and (height is not None):
        #    graph = adagenes.plot.generate_plots.generate_radarplot.generate_radar_plot(df_cat,names, "pathogenicity-scores-radar",width=width,height=height)
        #    return graph
        #else:
        #    return adagenes.plot.generate_plots.generate_radarplot.generate_radar_plot(dfs, names,
        #                                                                             "pathogenicity-scores-radar")
        return onkopus.onkopus_clients.PlotClient(genome_version="hg38").process_data(variant_data,plot="categorical_pathogenicity")
    elif (plot_type == "treatment-sunburst") \
            or (plot_type == "treatment-sunburst-cancer-type") \
            or (plot_type == "treatment-sunburst-match-type-drugs") \
            or (plot_type == "treatment-sunburst-match-type-drugs-all") \
            or (plot_type == "treatment-sunburst-match-type-drug-classes") \
            or (plot_type == "treatment-sunburst-response-type") \
            or (plot_type == "treatments-all-drug-class-drugs") \
            or (plot_type == "treatments-all-cancer-type") \
            or (plot_type == "treatments-all-sunburst-match-type-drug-classes-all") \
            or (plot_type == "treatments-all-sunburst-match-type-drugs-all") \
            or (plot_type == "treatments-all-sunburst-response-type") \
            or (plot_type == "treatments-all-patient-drugs") \
            or (plot_type == "treatments-all-patient-cancer-type") \
            or (plot_type == "treatments-all-patient-match-type-drug-classes") \
            or (plot_type == "treatments-all-patient-drug-class") \
            :
        return generate_sunburst_plot(variant_data, qid, plot_type, font_size=font_size, variant_type=mutation_type)
    elif plot_type == "chromosome-positions":
        ideogram_data = adagenes.generate_data.generate_chromosome_positions_data(variant_data, genome_version)
        return ideogram_data
    elif plot_type == "protein-data":
        graphJSON = onkopus.plot.generate_protein_plot_with_annotations(variant_data)
        return graphJSON
    elif plot_type == "pdb":
        if qid is not None:
            variant_data = variant_data[qid]
        graphJSON = onkopus.onkopus_clients.PDBClient(genome_version).process_data(variant_data)
        return graphJSON
    elif plot_type == "pdb_scannet":
        if qid is not None:
            graphJSON = onkopus.ScanNetBindingSiteClient(genome_version=genome_version).process_data(qid)
            return graphJSON
    elif plot_type == "pdb_openfold_wt":
        if qid is not None:
            variant_data = variant_data[qid]
        graphJSON = onkopus.onkopus_clients.PDBClient(genome_version).process_data(variant_data, plot=plot_type)
        return graphJSON
    elif plot_type == "pdb_openfold_mutated":
        if qid is not None:
            variant_data = variant_data[qid]
        graphJSON = onkopus.onkopus_clients.PDBClient(genome_version).process_data(variant_data, plot=plot_type)
        return graphJSON
    elif plot_type == "median_ge_per_tissue":
        graphJSON = op.GeneExpressionPlotClient().process_data(qid)
        return graphJSON

    return {}



import unittest, os
import adagenes
import onkopus.onkopus_clients


class TestOnkopusAnnotation(unittest.TestCase):

    def test_single_module_annotation(self):

        infile="../test_files/somaticMutations.vcf"
        outfile="../test_files/somaticMutations.vcf.onkopus.dbsnp"
        module="dbsnp"

        #adagenes.annotate(infile, outfile, module=module)

    def test_full_module_annotation(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.vcf"
        outfile = infile + ".annotated"
        genome_version='hg38'

        #adagenes.annotate_file_all_modules(infile, outfile, genome_version=genome_version, writer_output_format="json")

    def test_process_file_with_generic_transformer_vcf(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.vcf"
        outfile = infile + ".generic.vcf"
        genome_version = 'hg38'

        transformer = onkopus.onkopus_clients.DBSNPClient(genome_version)
        #adagenes.process_file(infile, outfile, transformer, genome_version=genome_version)

    def test_process_file_with_generic_transformer_json(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.vcf"
        outfile = infile + ".generic.json"
        genome_version = 'hg38'

        transformer = onkopus.onkopus_clients.DBSNPClient(genome_version)
        #adagenes.process_file(infile, outfile, transformer, genome_version=genome_version)



import traceback, datetime, json
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as conf_reader
import requests
from json import JSONDecodeError


class GeneExpressionPlotClient:

    def __init__(self, error_logfile=None):
        #self.genome_version = genome_version
        self.info_lines= conf_reader.onkopus_plots_info_lines
        self.url_pattern = conf_reader.onkopus_plots_src
        self.srv_prefix = conf_reader.onkopus_plots_srv_prefix
        self.response_keys = conf_reader.onkopus_plots_response_keys
        self.extract_keys = conf_reader.onkopus_plots_keys

        self.qid_key = "q_id"
        self.error_logfile = error_logfile

    def process_data(self, variant_data):
            """
            Generates JSON data for a Plotly variant needle plot with added variant annotations

            :param variant_data:
            :return:
            """
            if "UTA_Adapter" in variant_data:
                if ("gene_name" in variant_data["UTA_Adapter"]):

                    url = conf_reader.ge_plot_module_src + "?gene=" + variant_data["UTA_Adapter"]["gene_name"]

                    print(url)
                    graphJSON = requests.get(url, timeout=60)
                    try:
                        return graphJSON.json()
                    except JSONDecodeError:
                        print("Could not decode JSON: ", graphJSON,": ",traceback.format_exc())
                        return {}
            else:
                url = conf_reader.ge_plot_module_src + "?gene=" + variant_data

                print(url)
                graphJSON = requests.get(url, timeout=60)
                try:
                    return graphJSON.json()
                except JSONDecodeError:
                    print("Could not decode JSON: ", graphJSON, ": ", traceback.format_exc())
                    return {}



import unittest
import adagenes as ag
import onkopus as op


class IndelToGeneTestCase(unittest.TestCase):

    def test_uta_adapter_indeltogene_client(self):
        genome_version = 'hg38'
        data = {"chr16:68812175insTTCAA": {}}
        bframe = ag.BiomarkerFrame(data)
        print(bframe.data)
        variant_data = op.IndelToGeneClient(
            genome_version=genome_version).process_data(bframe.data)
        #variant_data = op.UTAAdapterClient(
        #    genome_version=genome_version).process_data(variant_data)
        #variant_data = op.CCSGeneToGenomicClient(
        #    genome_version=genome_version).process_data(variant_data)
        #print("Response ",variant_data["chr1:114713908T>A"]["UTA_Adapter"])
        print(bframe.data)
        qids = ["chr16:68812175insTTCAA"]
        self.assertListEqual(list(variant_data.keys()), qids, "Error UTA adapter IndelToGene")
        self.assertEqual(variant_data["chr16:68812175insTTCAA"]["UTA_Adapter_indel"]["gene_name"],"CDH1","")







import unittest, os
import onkopus as op

class TestOnkopusServer(unittest.TestCase):

    def test_onkopus_server_file_upload(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../../test_files/somaticMutations.ln50.vcf"
        outfile = __location__ + "/../../test_files/somaticMutations.ln50.revel.csv"
        client = op.OnkopusServerClient("hg38")
        client.interpret_variant_file(infile, "hg38")





import traceback, datetime, json
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as conf_reader
import requests
from json import JSONDecodeError


class PDBClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= conf_reader.onkopus_plots_info_lines
        self.url_pattern = conf_reader.onkopus_plots_src
        self.srv_prefix = conf_reader.onkopus_plots_srv_prefix
        self.response_keys = conf_reader.onkopus_plots_response_keys
        self.extract_keys = conf_reader.onkopus_plots_keys

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

    def process_data(self, variant_data, plot=None):
            """
            Generates JSON data for a Plotly variant needleplot with added variant annotations

            :param variant_data:
            :return:
            """
            if "UTA_Adapter" in variant_data:
                if ("gene_name" in variant_data["UTA_Adapter"]):# and ("variant_exchange" in variant_data["UTA_Adapter"]):
                    #q = variant_data["UTA_Adapter"]["gene_name"] + ':' + variant_data["UTA_Adapter"]["variant_exchange"]
                    # url = conf_reader.protein_module_src + q

                    if plot is not None:
                        if plot == "pdb_openfold_wt":
                            url = conf_reader.protein_module_pdb_openfold_wt_src + "/" + variant_data["UTA_Adapter"][
                                "gene_name"] + "/True"
                        elif plot == "pdb_openfold_mutated":
                            url = conf_reader.protein_module_pdb_openfold_mutated_src + "/" + variant_data["UTA_Adapter"][
                                "gene_name"] + ":" + variant_data["UTA_Adapter"][
                                "variant_exchange"] + "/True"
                        else:
                            url = conf_reader.protein_module_pdb_src + "/" + variant_data["UTA_Adapter"]["gene_name"] + "/True"
                    else:
                        url = conf_reader.protein_module_pdb_src + "/" + variant_data["UTA_Adapter"][
                            "gene_name"] + "/True"

                    print(url)
                    graphJSON = requests.get(url, timeout=60)
                    try:
                        return graphJSON.json()
                    except JSONDecodeError:
                        print("Could not decode JSON: ", graphJSON,": ",traceback.format_exc())
                        return {}
            else:
                print("Could not find UTA adapter section")
                return {}



import unittest
import onkopus.onkopus_clients

class MetaKBAnnotationTestCase(unittest.TestCase):

    def test_metakb_client(self):

        genome_version = 'hg38'
        qid = "chr7:140753336A>T"
        data = {qid: {}}
        data = onkopus.onkopus_clients.UTAAdapterClient(genome_version=genome_version).process_data(data)
        variant_data = onkopus.onkopus_clients.MetaKBClient(
            genome_version=genome_version).process_data(data)

        self.assertEqual(len(variant_data[qid]["metakb"]["metakb_features_norm"]["exact_match"]), 573,
                         "Error retrieving MetaKB response")




import unittest
import adagenes as ag
import onkopus as op


class TranscriptToGenomicTestCase(unittest.TestCase):

    def test_uta_adapter_transcripttogenomic_client(self):
        genome_version = 'hg38'
        data = {"NM_004985.5:c.35G>A": {}}
        bframe = ag.BiomarkerFrame(data)
        print(bframe.data)
        variant_data = op.TranscriptToGenomicClient(
            genome_version=genome_version).process_data(bframe.data)
        #variant_data = op.UTAAdapterClient(
        #    genome_version=genome_version).process_data(variant_data)
        #variant_data = op.CCSGeneToGenomicClient(
        #    genome_version=genome_version).process_data(variant_data)
        #print("Response ",variant_data["chr1:114713908T>A"]["UTA_Adapter"])
        print(bframe.data)
        qids = ['chr12:25245350C>T']
        self.assertListEqual(list(variant_data.keys()), qids, "Error UTA adapter GeneToGenomic")
        self.assertEqual(variant_data['chr12:25245350C>T']["mutation_type"],"snv","")

    def test_transcripttogenomic_client1(self):
        genome_version = 'hg38'
        data = {'NM_004985.5:C.35G>A': {'type': 'c', 'mutation_type': 'snv', 'mdesc': 'transcript_cdna'}}
        bframe = ag.BiomarkerFrame(data)
        print("start ",bframe)
        variant_data = op.TranscriptToGenomicClient(
            genome_version=genome_version).process_data(bframe.data)
        print("ok ",bframe.data)
        qids = ['chr12:25245350C>T']
        self.assertListEqual(list(variant_data.keys()), qids, "Error UTA adapter GeneToGenomic")
        self.assertEqual(variant_data['chr12:25245350C>T']["mutation_type"], "snv", "")





from onkopus.onkopus_clients import CCSGeneFusionClient
from adagenes.tools import generate_keys
from onkopus.conf import read_config as conf_reader
import onkopus as op


def annotate_fusions(annotated_data, genome_version="hg38",
                     oncokb_key=None, tumor_type=None,
                     request_id=None):
    snv_data = {}
    for var in annotated_data.keys():
        mut_type = ""
        if "variant_data" in annotated_data[var]:
            if "mutation_type" in annotated_data[var]["variant_data"]:
                mut_type = annotated_data[var]["variant_data"]["mutation_type"]
        elif "mutation_type" in annotated_data[var]:
            mut_type = annotated_data[var]["mutation_type"]
        if mut_type == "fusion":
            snv_data[var] = annotated_data[var]

    if len(list(snv_data.keys())) > 0:
        snv_data = annotate_fusion_data(snv_data, genome_version=genome_version,tumor_type=tumor_type)
        snv_data = op.AggregatorClient(genome_version=genome_version).process_data(snv_data)

    for var in snv_data.keys():
        annotated_data[var] = snv_data[var]

    #return annotated_data
    return snv_data

def fusion_request(gene_fusion_str: str, genome_version='hg38', tumor_type=None):
    """
    Annotates gene fusions with associated Onkopus modules

    :param gene_fusion_str:
    :param genome_version:
    :return:
    """
    print("gene fusion query: ", gene_fusion_str)
    if gene_fusion_str == "":
        return {}
    annotated_data = {}
    if gene_fusion_str != "":
        annotated_data[gene_fusion_str] = {}

    annotated_data = annotate_fusion_data(annotated_data, genome_version=genome_version, tumor_type=tumor_type)

    return annotated_data


def annotate_fusion_data(annotated_data, genome_version="hg38", tumor_type=None):
    """

    :param annotated_data:
    :param genome_version:
    :param tumor_type:
    :return:
    """
    client = CCSGeneFusionClient(
        genome_version=genome_version)
    annotated_data = client.process_data(annotated_data)
    annotated_data = generate_keys(annotated_data, conf_reader.onkopus_modules)

    return annotated_data



import unittest, copy
import onkopus.onkopus_clients

class DrugClassAnnotationTestCase(unittest.TestCase):

    def test_drugclass_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr17:7673776G>A": {}}

        variant_data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)

        variant_data = onkopus.onkopus_clients.MetaKBClient(
            genome_version=genome_version).process_data(variant_data)

        variant_data = onkopus.onkopus_clients.CIViCClient(
            genome_version=genome_version).process_data(variant_data)

        variant_data = onkopus.onkopus_clients.OncoKBClient(
            genome_version=genome_version).process_data(variant_data)

        variant_data = onkopus.onkopus_clients.AggregatorClient(genome_version).process_data(variant_data)

        variant_data = onkopus.onkopus_clients.DrugOnClient(
            genome_version=genome_version).process_data(variant_data)

        #print("Response ",variant_data)
        for var in variant_data.keys():
            print(variant_data[var]["drug_classes"])
        print(variant_data["chr7:140753336A>T"]["onkopus_aggregator"]["merged_match_types_data"][0]["drugs"])
        self.assertListEqual(variant_data["chr7:140753336A>T"]["onkopus_aggregator"]["merged_match_types_data"][0]["drugs"][1]["drug_class"],['ANTINEOPLASTIC--PI3K Inhibitor'],"")




import os
import subprocess
import onkopus.conf.read_config as conf_reader
import onkopus.conf.docker_config as docker_config


def get_docker_compose_files(module):
    if module in docker_config.module_ids:
        return docker_config.module_ids[module]
    else:
        return []


def _add_new_module(module, __data_dir__):
    with open(__data_dir__ + "/" + conf_reader.config["DEFAULT"]["INSTALLED_MODULES_FILE"], "a") as file:
        file.write(module + '\n')


def install(module, installed_modules, available_modules):
    """
    Installs an Onkopus module

    :param module:
    :return:
    """
    __location__ = os.path.realpath(
        os.path.join(os.getcwd(), os.path.dirname(__file__)))
    data_dir = __location__ + "/../conf"

    print("Install Onkopus module: ", module)

    # Check if module is already installed
    #if module in installed_modules:
    #    print("Module " + module + " is already installed")
    #    return

    # Download databases
    if module in available_modules:
        # add Docker compose entry
        #print("Adding Docker compose entry for " + module + "...", end='')
        template_files = get_docker_compose_files(module) #[data_dir + "/docker_compose/" + module + ".yml"]
        for tfile in template_files:
            os.system("cp -v " + data_dir + "/docker_compose/" + tfile + " " + data_dir + "/installed_modules/")

        #if module == "uta-adapter":
        #    template_files.append(data_dir + "/docker_compose/uta-database.yml")

        #for tfile in template_files:
        #    template_file = open(tfile)
        #    lines = []
        #    for line in template_file:
        #        lines.append(line)

        #    #docker_compose_file = data_dir + "/data/docker-compose.yml"
        #    #with open(docker_compose_file, "a") as dc_file:
        #    #    for line in lines:
        #    #        dc_file.write(line)
        #    #template_file.close()

        # pull Onkopus module Docker container
        #print("Pull Docker container for " + module, end='')
        print("Pull Docker container for " + module)
        #cmd = "docker-compose -f (ls " + data_dir + "/installed_modules | tr '\n' ' ') pull"
        #f"ls {data_dir}/installed_modules/*.yml | tr '\n' ' '"

        list_command = f"ls {data_dir}/installed_modules/*.yml | tr '\n' ' '"

        # Execute the list command and capture the output
        result = subprocess.run(list_command, shell=True, capture_output=True, text=True)

        # Check if the command was successful
        if result.returncode == 0:
            # Get the list of .yml files as a single string with spaces
            yml_files = result.stdout.strip()

            file_list = yml_files.split()
            prefixed_files = ['-f ' + file for file in file_list]
            prefixed_files_str = ' '.join(prefixed_files)

            # Construct the docker-compose command
            cmd = f"docker-compose "
            cmd += prefixed_files_str
            #for yml_file in yml_files:
            #    cmd += f"-f {yml_file} "
            cmd += " pull"

            print(cmd)
            # os.system("docker-compose " + data_dir + "/data/docker-compose.yml pull")
            #os.system(cmd)

            # Execute the docker-compose command
            subprocess.run(cmd, shell=True)
        else:
            print(f"Error listing .yml files: {result.stderr}")

        _add_new_module(module, data_dir)

        print("Onkopus module successfully installed: " + module)
    else:
        print(
            "[Error] No Onkopus module found: " + module + ". Get a list of all available modules with 'onkopus list-modules'")

def start_modules(installed_modules):
        """
        Starts all installed Onkopus modules as Docker containers

        :return:
        """
        print("Starting locally installed Onkopus modules")
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        data_dir = __location__ + "/../conf"

        print("Installed modules: " + ",".join(installed_modules))

        # Start Docker containers
        #dc_cmd = "docker-compose -f " + data_dir + "/data/docker-compose.yml up -d"
        #print(dc_cmd)
        #os.system(dc_cmd)
        list_command = f"ls {data_dir}/installed_modules/*.yml | tr '\n' ' '"

        # Execute the list command and capture the output
        result = subprocess.run(list_command, shell=True, capture_output=True, text=True)

        # Check if the command was successful
        if result.returncode == 0:
            # Get the list of .yml files as a single string with spaces
            yml_files = result.stdout.strip()
            file_list = yml_files.split()
            prefixed_files = ['-f ' + file for file in file_list]
            prefixed_files_str = ' '.join(prefixed_files)

            # Construct the docker-compose command
            #cmd = f"docker-compose -f {yml_files} up -d"
            cmd = f"docker-compose "
            cmd += prefixed_files_str
            #for yml_file in yml_files:
            #    cmd += f"-f {yml_file} "
            cmd += " up -d"

            print(cmd)
            # os.system("docker-compose " + data_dir + "/data/docker-compose.yml pull")
            # os.system(cmd)

            # Execute the docker-compose command
            subprocess.run(cmd, shell=True)
        else:
            print(f"Error listing .yml files: {result.stderr}")

        print("Onkopus started")

def stop_modules():
        """
        Stops all running Onkopus containers

        :return:
        """
        print("Stopping Onkopus modules")
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        data_dir = __location__ + "/../conf"

        #os.system("docker-compose -f " + data_dir + "/docker-compose.yml down")
        list_command = f"ls {data_dir}/installed_modules/*.yml | tr '\n' ' '"

        # Execute the list command and capture the output
        result = subprocess.run(list_command, shell=True, capture_output=True, text=True)

        # Check if the command was successful
        if result.returncode == 0:
            # Get the list of .yml files as a single string with spaces
            yml_files = result.stdout.strip()
            file_list = yml_files.split()
            prefixed_files = ['-f ' + file for file in file_list]
            prefixed_files_str = ' '.join(prefixed_files)

            # Construct the docker-compose command
            #cmd = f"docker-compose -f {yml_files} stop"
            cmd = f"docker-compose "
            cmd += prefixed_files_str
            #for yml_file in yml_files:
            #    cmd += f"-f {yml_file} "
            cmd += " stop"

            print(cmd)
            # os.system("docker-compose " + data_dir + "/data/docker-compose.yml pull")
            # os.system(cmd)

            # Execute the docker-compose command
            subprocess.run(cmd, shell=True)
        else:
            print(f"Error listing .yml files: {result.stderr}")

        print("Onkopus stopped")



from onkopus.conf import *
from .onkopus_clients import *
from .onkopus_writer import *
from .processing import annotate_snvs,annotate_variant_data,get_onkopus_client,indel_request,annotate_genes,identify_genes_from_request, annotate,annotate_indels,annotate_fusions,analyze_snv_request,fusion_request,analyze_genomic_location_request
from .onkopus_main import *
from .tools import *
from .convert import *




###
# Main script for running the Onkopus biomarker interpretation framework
#
# Author: Nadine S. Kurz
#
###
import onkopus.onkopus_main

if __name__ == '__main__':
    onkopus = onkopus.onkopus_main.OnkopusMain()
    onkopus.run()





import copy
import adagenes
import adagenes.conf.read_config as config

def split_data_by_mutation_type(data):

    dc = {"snvs": {}, "indels": {}, "fusions": {}, "genes": {},"unidentified": {}}

    for var in data.keys():
        if "mutation_type" in data[var]:
            if data[var]["mutation_type"] == "snv":
                dc["snvs"][var] = data[var]
            elif data[var]["mutation_type"] == "indel":
                dc["indels"][var] = data[var]
            elif data[var]["mutation_type"] == "fusion":
                dc["fusions"][var] = data[var]
            elif data[var]["mutation_type"] == "gene":
                dc["genes"][var] = data[var]
            else:
                dc["unidentified"][var] = data[var]
        else:
            dc["unidentified"][var] = data[var]

    return dc

def get_genomic_qid(data):

    #if "hgnc_symbol"
    pass
    # qid


def protein_to_genomic(annotated_data, results_string, res, vcf_lines,module, genome_version="hg38"):
    """
    Converts data of a single biomarker including IDs on protein level into IDs on genomic level

    :param annotated_data:
    :param results_string:
    :return:
    """
    chr, ref_seq, pos, ref, alt = adagenes.tools.parse_genomic_data.parse_genome_position(
        results_string)
    qid = 'chr' + chr + ':' + pos + ref + '>' + alt

    if qid not in annotated_data:
        annotated_data[qid] = copy.deepcopy(vcf_lines)

    if config.variant_data_key not in annotated_data[qid]:
        annotated_data[qid][config.variant_data_key] = {}

    annotated_data[qid][config.variant_data_key]['CHROM'] = chr
    annotated_data[qid][config.variant_data_key]['reference_sequence'] = ref_seq
    annotated_data[qid][config.variant_data_key]['POS'] = pos
    annotated_data[qid][config.variant_data_key]['REF'] = ref
    annotated_data[qid][config.variant_data_key]['ALT'] = alt
    annotated_data[qid][config.variant_data_key]['POS_' + genome_version] = pos
    annotated_data[qid]["q_id"] = "chr" + chr + ":" + str(pos) + ref + ">" + alt
    annotated_data[qid][config.variant_data_key]['ID'] = ''
    annotated_data[qid][config.variant_data_key]['QUAL'] = ''
    annotated_data[qid][config.variant_data_key]['FILTER'] = ''
    annotated_data[qid][config.variant_data_key]['type'] = 'g'
    annotated_data[qid]["mutation_type_detail"] = 'Missense_Mutation'

    if "refAmino" in res:
        annotated_data[qid][config.variant_data_key]['ref_aa'] = res["refAmino"]
        annotated_data[qid][config.variant_data_key]['alt_aa'] = res["varAmino"]

    if module is not None:
        annotated_data[qid][module] = res
    else:
        annotated_data[qid] = res

    # add previously known information on gene names and aa exchange
    if "hgnc_symbol" in res:
        annotated_data[qid][config.variant_data_key]["gene_name"] = res['hgnc_symbol']
        annotated_data[qid][config.variant_data_key]["amino_acid_exchange"] = res['aminoacid_exchange']
        if "UTA_Adapter" not in annotated_data[qid].keys():
            annotated_data[qid]["UTA_Adapter"] = {}
        annotated_data[qid]["UTA_Adapter"]["gene_name"] = res['hgnc_symbol']
        annotated_data[qid]["UTA_Adapter"]["variant_exchange"] = res['aminoacid_exchange']

    annotated_data[qid][config.uta_adapter_genetogenomic_srv_prefix] = res
    annotated_data[qid]["level"] = "g"
    annotated_data[qid]["type"] = "g"

    return annotated_data

def transcript_to_genomic(annotated_data, results_string, res, vcf_lines,module, genome_version="hg38"):
    """
    Converts data of a single biomarker including IDs on protein level into IDs on genomic level

    :param annotated_data:
    :param results_string:
    :return:
    """
    #nm, ref_seq, pos, ref, alt = adagenes.tools.parse_genomic_data.parse_transcript_cdna(
    #    results_string)
    #print("res ",res)
    variant_g = res["variant_g"]
    chr, ref_seq, pos, ref, alt = adagenes.tools.parse_genomic_data.parse_genome_position(variant_g)
    #qid = 'chr' + chrom + ':' + pos + ref + '>' + alt

    #print("elements ",chr,ref_seq, pos, ref, alt)
    qid = 'chr' + chr + ':' + pos + ref + '>' + alt

    if qid not in annotated_data:
        annotated_data[qid] = copy.deepcopy(vcf_lines)

    if config.variant_data_key not in annotated_data[qid]:
        annotated_data[qid][config.variant_data_key] = {}

    annotated_data[qid][config.variant_data_key]['CHROM'] = chr
    annotated_data[qid][config.variant_data_key]['reference_sequence'] = ref_seq
    annotated_data[qid][config.variant_data_key]['POS'] = pos
    annotated_data[qid][config.variant_data_key]['REF'] = ref
    annotated_data[qid][config.variant_data_key]['ALT'] = alt
    annotated_data[qid][config.variant_data_key]['POS_' + genome_version] = pos
    annotated_data[qid]["q_id"] = "chr" + chr + ":" + str(pos) + ref + ">" + alt
    annotated_data[qid][config.variant_data_key]['ID'] = ''
    annotated_data[qid][config.variant_data_key]['QUAL'] = ''
    annotated_data[qid][config.variant_data_key]['FILTER'] = ''
    annotated_data[qid][config.variant_data_key]['type'] = 'g'
    annotated_data[qid]["mutation_type_detail"] = 'Missense_Mutation'

    if "refAmino" in res:
        annotated_data[qid][config.variant_data_key]['ref_aa'] = res["refAmino"]
        annotated_data[qid][config.variant_data_key]['alt_aa'] = res["varAmino"]

    if module is not None:
        annotated_data[qid][module] = res
    else:
        annotated_data[qid] = res

    # add previously known information on gene names and aa exchange
    if "hgnc_symbol" in res:
        annotated_data[qid][config.variant_data_key]["gene_name"] = res['hgnc_symbol']
        annotated_data[qid][config.variant_data_key]["amino_acid_exchange"] = res['aminoacid_exchange']
        if "UTA_Adapter" not in annotated_data[qid].keys():
            annotated_data[qid]["UTA_Adapter"] = {}
        annotated_data[qid]["UTA_Adapter"]["gene_name"] = res['hgnc_symbol']
        annotated_data[qid]["UTA_Adapter"]["variant_exchange"] = res['aminoacid_exchange']

    annotated_data[qid][config.uta_adapter_genetogenomic_srv_prefix] = res
    annotated_data[qid]["level"] = "g"
    annotated_data[qid]["type"] = "g"

    return annotated_data
